# $Id: jsdl_filter.py 2588 2009-07-28 11:03:09Z mmamonski $
#
# vim: set ts=4 sw=4 et:

#
# QosCosGrid Computing
# Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

try:
    # Try using faster implementation in C.
    import xml.etree.cElementTree as etree
except:
    import xml.etree.ElementTree as etree

from qcg.core import *
from qcg.comp import *

class Module(Module):

    def __init__(self, *unused):
        signal_connect("smc_jsdl_filter", self.signal_handler)

    def signal_handler(self, signal, original_jsdl, chained_params):

        # The previous signal might have used signal->stop_emission but he
        # didn't have to and there's no use processing if there was a failure.
        # This step might be ignored but beware that even if there was a failure
        # in the processing chain, the last returned value will be eventually
        # used. That's why signal->stop_emission is a good idea if a handler
        # fails and one wants to be sure that the error is propagated up.
        if previous_signal_failed(chained_params):
            return chained_params

        # If we happen to be the first handler called, we need to use the source
        # parameters.
        elif signal_first(chained_params):
            jsdl = original_jsdl

        # We should work on the values produced by the previous handler
        # (chained_params contains the return value from the last handler). If
        # we had used original_jsdl here we would have ignored the work of the
        # previous handlers!
        else:
            jsdl = chained_params[1]

        #
        # TODO: Modify jsdl here.
        #
        # It's a good idea to call signal.stop_emission() if we are returning
        # something other than JSDL_FILTER_OK.
        #

        return (RetCode(JSDL_FILTER_OK), jsdl)
