# $Id: factory_attr.py 2588 2009-07-28 11:03:09Z mmamonski $
#
# vim: set ts=4 sw=4 et:

#
# QosCosGrid Computing
# Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

try: 
  from qcg.core import *
  from qcg.comp import *
except ImportError:
  class Module:
    pass
  def signal_connect(a1,a2):
    pass

import subprocess 
from xml.dom.minidom import Document
import os
import re
import sys

class ModulesInfoModule(Module):
  
  def __init__(self, *unused):
    signal_connect("smc_factory_attributes", self.signal_handler)
    
    self.__QCGCompFactoryNS = "http://schemas.qoscosgrid.org/comp/2011/04/factory"
 

  def decodeUnicode(self, to_decode):
    if sys.version_info[0] >= 3:
      return str(to_decode.decode('ascii','ignore'))

    return to_decode


  def module_avail(self):
    try:
      sr = subprocess.Popen(args = ["-c", ". /etc/profile && module avail"], executable="/bin/bash", stderr=subprocess.STDOUT, stdout=subprocess.PIPE, shell=True)
    except OSError as e:
      raise Exception("module avail command failed, reason:\n" + str(e))

    output = sr.communicate()[0]
    if sr.returncode != 0:
      raise Exception("module avail command returned code: " + str(sr.returncode) + ". Details: " + output)

    return self.decodeUnicode(output)


  def signal_handler(self, signal, local_resource_manager_type, not_used):
    command_output = ""
    
    try:
        command_output = self.module_avail()
    except subprocess.CalledProcessError as e:
        print("[ERROR] - failed to get module list: ", e.output)

    doc = Document()    
    modulesEl = doc.createElementNS(self.__QCGCompFactoryNS, "Modules")    
    modulesEl.setAttribute("xmlns",self.__QCGCompFactoryNS)
    doc.appendChild(modulesEl)
    
    for token in command_output.split():
      if token.startswith("-") or token.startswith("/"):
        continue
      if token.find("/") == -1:
        continue

      module_name = token.rsplit("/", 1)[0]
      module_version = token.rsplit("/", 1)[1]
      
      moduleEl = doc.createElement("Module")
      moduleEl.setAttribute("name", module_name)
      moduleEl.setAttribute("version", module_version)
      modulesEl.appendChild(moduleEl)
    
    
    result = doc.toxml().replace("<?xml version=\"1.0\" ?>", "")
    doc.unlink()
    return result


if __name__ == '__main__':
    plgrid = ModulesInfoModule()

    doc = plgrid.signal_handler(None, None, None)
    print(doc)
