# $Id$
#
# vim: set ts=4 sw=4 et:

#
# QosCosGrid Computing
# Copyright (C) 2008-2017 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

try: 
  from qcg.core import *
  from qcg.comp import *
except ImportError:
  class Module:
    pass
  def signal_connect(a1,a2):
    pass

from xml.dom.minidom import Document
import os
import re


class NodeTypesModule(Module):
  
  def __init__(self, *unused):
    signal_connect("smc_factory_attributes", self.signal_handler)
    self.__QCGCompFactoryNS = "http://schemas.qoscosgrid.org/comp/2011/04/factory"
#    self.__NodesFile = "./nodes.conf"
    self.__NodesFile = "/etc/qcg/qcg-comp/nodes.conf"
    self.__NodesFileRegex = re.compile(r'^[\t ]*([^# \t]+)[\t ]+([^#]+)\n')
    self.__nodes = {}

  def signal_handler(self, signal, local_resource_manager_type, not_used):
    self.__refresh()

    doc = Document()    
    nodesEl = doc.createElementNS(self.__QCGCompFactoryNS, "NodeTypes")
    nodesEl.setAttribute("xmlns",self.__QCGCompFactoryNS)
    doc.appendChild(nodesEl)
    
    for ntype in self.__nodes:
      nprops = self.__nodes[ntype]
      el = doc.createElement("NodeType")

      nid = doc.createElement("Id")
      nidText = doc.createTextNode(ntype)
      nid.appendChild(nidText)

      el.appendChild(nid)

      np = doc.createElement("Attributes")
      for attr in nprops.split(","):
          npAttr = doc.createElement("Attribute")
          npAttrText = doc.createTextNode(attr.strip())
          npAttr.appendChild(npAttrText)
          np.appendChild(npAttr)

      el.appendChild(np)

      nodesEl.appendChild(el)

    result = doc.toxml().replace("<?xml version=\"1.0\" ?>", "")
    doc.unlink()
    return result

  def __refresh(self):
    if os.path.exists(self.__NodesFile):
#    if os.path.exists(self.__VOFile) and os.path.getmtime(self.__VOFile) != self.__vOFileMTime:
#        self.__voFileMTime = os.path.getmtime(self.__VOFile)
      self.__load_file()

  def __load_file(self):
    f = open(self.__NodesFile, 'r')
#    print "opened file ",self.__NodesFile
    self.__nodes = {}
    for line in f:
#      print "read line (%s)" % (line)
      m = self.__NodesFileRegex.match(line)
      if m != None:
        nid = m.group(1).strip()
        nprops = m.group(2).strip()

        if nid:
          self.__nodes[nid] = nprops
      
    f.close()


if __name__ == '__main__':
    _types = NodeTypesModule()

    doc = _types.signal_handler(None, None, None)
    
    print(doc)
