# $Id: factory_attr.py 2588 2009-07-28 11:03:09Z mmamonski $
#
# vim: set ts=4 sw=4 et:

#
# QosCosGrid Computing
# Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

try: 
  from qcg.core import *
  from qcg.comp import *
except ImportError:
  class Module:
    pass
  def signal_connect(a1,a2):
    pass

from xml.dom.minidom import Document
import os
import re


class UserInfo(object):
  "User information storage class"
  
  def __init__(self):
    self.__username = None
    self.__DNs = []
    self.__groups = []
    self.__grants = []
    self.__scratch_dir = None
    self.__private_grant = None
    self.__default_grant = None
    
  def __getUserName(self):
    return self.__username
  
  def __setUserName(self, value):
    if not isinstance(value, str):
      raise TypeError("UserName must be a string")
    self.__username = value

  def __getScratchDir(self):
    return self.__scratch_dir
  
  def __setScratchDir(self, value):
    if not isinstance(value, str):
      raise TypeError("ScratchDir must be a string")
    self.__scratch_dir = value

  def __getDNs(self):
    return self.__DNs
  
  def __setDNs(self, value):
    if not isinstance(value, list):
      raise TypeError("DNs must be a list")
    self.__groups = value

  def __getGroups(self):
    return self.__groups
  
  def __setGroups(self, value):
    if not isinstance(value, list):
      raise TypeError("Groups must be a list")
    self.__groups = value

  def __getGrants(self):
    return self.__grants
  
  def __setGrants(self, value):
    if not isinstance(value, list):
      raise TypeError("Grants must be a list")
    self.__grants = value

  def __getPrivateGrant(self):
    return self.__private_grant
  
  def __setPrivateGrant(self, value):
    if not isinstance(value, str):
      raise TypeError("PrivateGrant must be a string")
    self.__private_grant = value

  def __getDefaultGrant(self):
    return self.__default_grant
  
  def __setDefaultGrant(self, value):
    if not isinstance(value, str):
      raise TypeError("DefaultGrant must be a string")
    self.__default_grant = value
  
  UserName = property(__getUserName, __setUserName, None, \
                 "String, gets/sets user name")
  
  ScratchDir = property(__getScratchDir, __setScratchDir, None, \
          "String, gets/sets ScratchDir")
  
  DNs = property(__getDNs, __setDNs, None, \
             "DateTime, gets/sets DNs")
  
  Groups = property(__getGroups, __setGroups, None, \
             "DateTime, gets/sets groups")
  
  Grants = property(__getGrants, __setGrants, None, \
           "String, gets/sets grants")
  
  PrivateGrant = property(__getPrivateGrant, __setPrivateGrant, None, \
           "String, gets/sets private grants")
  
  DefaultGrant = property(__getDefaultGrant, __setDefaultGrant, None, \
           "String, gets/sets default grants")


class PLInfoModule(Module):
  
  def __init__(self, *unused):
    signal_connect("smc_factory_attributes", self.signal_handler)
    
    self.__QCGCompFactoryNS = "http://schemas.qoscosgrid.org/comp/2011/04/factory"
    self.__QCGScratchDirRoot = os.getenv("QCG_SCRATCH_DIR_ROOT")
    self.__QCGGroupsDirRoot = os.getenv("QCG_GROUP_DIR_ROOT")
    if os.getenv("QCG_STATE_DIR") is not None:
        self.__stateDir = os.getenv("QCG_STATE_DIR")
    else:
        self.__stateDir = "/var/run/qcg/qcg-comp/"
    self.__GridMapfile = "/etc/grid-security/grid-mapfile"
    self.__GroupsFile = self.__stateDir + "plgrid-groups"
    self.__GrantsFile = self.__stateDir + "plgrid-group.grants"
    self.__PrivateGrantsFile = self.__stateDir + "plgrid-private.grants"
    self.__DefaultGrantsFile = self.__stateDir + "plgrid-default.grants"
    self.__mapfileRegex = re.compile(r'\"(.+)\"[\t ]+([^\t\n ]+)[\t ]*([^\t ]+)?\n')
  
  def load_gridmap_file(self, users):
    f = open(self.__GridMapfile, 'r')
    for line in f:
      m = self.__mapfileRegex.match(line)
      if m != None:
        dn = m.group(1)
        userId = m.group(2)
        
        if userId not in users:
          user = UserInfo()
          user.UserName = userId
          users[userId] = user

        users[userId].DNs.append(dn)
      
    f.close()
  
  def load_groups_file(self, users, groups):
    f = open(self.__GroupsFile, 'r')
    for line in f:
      tuple = line.split()
      if len(tuple) == 2:
        group = tuple[0]
        userId = tuple[1]
        if userId in users:
          users[userId].Groups.append(group)
          if group not in groups:
            groups[group] = [] 
      
    f.close()
  
  def load_grants_file(self, groups):
    f = open(self.__GrantsFile, 'r')
    for line in f:
      tuple = line.split()
      if len(tuple) == 2:
        group = tuple[0]
        grant = tuple[1]
        
        if group not in groups:
          groups[group] = []
        
        groups[group].append(grant)
      
    f.close()

  def load_private_grants_file(self, users):
    f = open(self.__PrivateGrantsFile, 'r')
    for line in f:
      tuple = line.split()
      if len(tuple) == 2:
        grant = tuple[0]
        userId = tuple[1]
        if userId in users:
          users[userId].PrivateGrant = grant
      
    f.close()

  def load_default_grants_file(self, users):
    f = open(self.__DefaultGrantsFile, 'r')
    for line in f:
      tuple = line.split()
      if len(tuple) == 2:
        grant = tuple[0]
        userId = tuple[1]
        if userId in users:
          users[userId].DefaultGrant = grant
      
    f.close()

  def signal_handler(self, signal, local_resource_manager_type, not_used):
    users = dict()
    groups = dict()
    
    #load data
    self.load_gridmap_file(users)
    self.load_groups_file(users, groups)
    self.load_grants_file(groups)
    self.load_private_grants_file(users)
    self.load_default_grants_file(users)
  
    doc = Document()    
    plgInfoEl = doc.createElementNS(self.__QCGCompFactoryNS, "PlGridInfo")    
    plgInfoEl.setAttribute("xmlns",self.__QCGCompFactoryNS)
    doc.appendChild(plgInfoEl)

    usersEl = doc.createElementNS(self.__QCGCompFactoryNS, "Users")    
    plgInfoEl.appendChild(usersEl)
    
    for username in users.keys():
      user = users[username]
      
      userEl = doc.createElement("User")
      userEl.setAttribute("name", username)
      
      for dn in user.DNs: 
        dnEl = doc.createElement("DN")
        dnElText = doc.createTextNode(dn)
        dnEl.appendChild(dnElText)
        userEl.appendChild(dnEl)
      
      for group in user.Groups: 
        groupEl = doc.createElement("Group")
        groupElText = doc.createTextNode(group)
        groupEl.appendChild(groupElText)
        userEl.appendChild(groupEl)
      
      for grant in user.Grants: 
        grantEl = doc.createElement("Grant")
        grantElText = doc.createTextNode(grant)
        grantEl.appendChild(grantElText)
        userEl.appendChild(grantEl)
        
      if self.__QCGScratchDirRoot != None:
        scratchDir = self.__QCGScratchDirRoot + "/" +  username
        #if os.path.isdir(scratchDir):
        scratchEl = doc.createElement("ScratchDir")
        scratchElText = doc.createTextNode(scratchDir)
        scratchEl.appendChild(scratchElText)
        userEl.appendChild(scratchEl)

      if user.PrivateGrant != None:
        grantEl = doc.createElement("PrivateGrant")
        grantElText = doc.createTextNode(user.PrivateGrant)
        grantEl.appendChild(grantElText)
        userEl.appendChild(grantEl)

      if user.DefaultGrant != None: 
        grantEl = doc.createElement("DefaultGrant")
        grantElText = doc.createTextNode(user.DefaultGrant)
        grantEl.appendChild(grantElText)
        userEl.appendChild(grantEl)
    
      usersEl.appendChild(userEl)
    
    groupsEl = doc.createElementNS(self.__QCGCompFactoryNS, "Groups")    
    plgInfoEl.appendChild(groupsEl)
    
    for groupName in groups.keys():
      grants = groups[groupName]
      
      groupEl = doc.createElement("Group")
      groupEl.setAttribute("name", groupName)
      
      for grant in grants: 
        grantEl = doc.createElement("Grant")
        grantElText = doc.createTextNode(grant)
        grantEl.appendChild(grantElText)
        groupEl.appendChild(grantEl)
      
      if self.__QCGGroupsDirRoot != None:
        groupDir = self.__QCGGroupDirRoot + "/" +  groupName
        if os.path.isdir(groupDir):
          dirEl = doc.createElement("GroupDir")
          dirElText = doc.createTextNode(dirDir)
          dirEl.appendChild(dirElText)
          groupEl.appendChild(dirEl)
      
      groupsEl.appendChild(groupEl)
    
    result = doc.toxml().replace("<?xml version=\"1.0\" ?>", "")
    doc.unlink()
    return result


if __name__ == '__main__':
    plgrid = PLInfoModule()

    doc = plgrid.signal_handler(None, None, None)
    print(doc)
