# $Id: vo_info.py 1048 2014-12-02 14:43:23Z pkopta $
#
# vim: set ts=4 sw=4 et:

#
# QosCosGrid Computing
# Copyright (C) 2008-2014 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

try: 
  from qcg.core import *
  from qcg.comp import *
except ImportError:
  class Module:
    pass
  def signal_connect(a1,a2):
    pass

from commands import getstatusoutput
from xml.dom.minidom import Document
import os
import re


class VOInfoModule(Module):
  
  def __init__(self, *unused):
    signal_connect("smc_factory_attributes", self.signal_handler)
    self.__QCGCompFactoryNS = "http://schemas.qoscosgrid.org/comp/2011/04/factory"
    self.__VOFile = "/etc/qcg/qcg-comp/vo.conf"
    self.__VOFileRegex = re.compile(r'^[\t ]*([^#]+)\n')
    self.__vos = []
#    self.__voFileMTime = 0

  def signal_handler(self, signal, local_resource_manager_type, not_used):
    self.__refresh()

    doc = Document()    
    voEl = doc.createElementNS(self.__QCGCompFactoryNS, "AuthorizedVO")
    voEl.setAttribute("xmlns",self.__QCGCompFactoryNS)
    doc.appendChild(voEl)
    
    for token in self.__vos:
      el = doc.createElement("VO")
      elText = doc.createTextNode(token)
      el.appendChild(elText)
      voEl.appendChild(el)

    result = doc.toxml().replace("<?xml version=\"1.0\" ?>", "")
    doc.unlink()
    return result

  def __refresh(self):
    if os.path.exists(self.__VOFile):
#    if os.path.exists(self.__VOFile) and os.path.getmtime(self.__VOFile) != self.__vOFileMTime:
#        self.__voFileMTime = os.path.getmtime(self.__VOFile)
      self.__load_file()

  def __load_file(self):
    f = open(self.__VOFile, 'r')
    self.__vos = []
    for line in f:
      m = self.__VOFileRegex.match(line)
      if m != None:
        vo = m.group(1).strip()

        if vo:
          self.__vos.append(vo)
      
    f.close()
