-- $Id: qcg-comp-psql.sql 2588 2009-07-28 11:03:09Z mmamonski $

--
-- QosCosGrid Computing
-- Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--

DROP TABLE states CASCADE;
DROP TABLE jobs CASCADE;
DROP TABLE jobs_acc CASCADE;
DROP TABLE reservations CASCADE;
DROP TABLE reservations_acc CASCADE;
DROP TABLE accounts_pool CASCADE;

CREATE TABLE states
(
	name VARCHAR(16) PRIMARY KEY
);

INSERT INTO states VALUES ('Stage-in');
INSERT INTO states VALUES ('Queued');
INSERT INTO states VALUES ('Held');
INSERT INTO states VALUES ('Executing');
INSERT INTO states VALUES ('Suspended');
INSERT INTO states VALUES ('Stage-out');
INSERT INTO states VALUES ('Finished');
INSERT INTO states VALUES ('Failed');
INSERT INTO states VALUES ('Cancelled');

CREATE TABLE jobs
(
	serial SERIAL PRIMARY KEY,

	id VARCHAR(36) NOT NULL UNIQUE,
	drms_id VARCHAR(255) UNIQUE,

	reservation_id VARCHAR(36),

	submitted_jsdl TEXT NOT NULL,
	jsdl TEXT NOT NULL,

	subject TEXT NOT NULL,
	mapped_user VARCHAR(255) NOT NULL,

	state VARCHAR(16) NOT NULL REFERENCES states(name),
	staging_fault TEXT,

	submission_time TIMESTAMP WITH TIME ZONE NOT NULL,
	start_time TIMESTAMP WITH TIME ZONE,
	end_time TIMESTAMP WITH TIME ZONE,
	CONSTRAINT valid_start_end CHECK (end_time >= start_time),

	exit_status INTEGER,
	core_dumped BOOLEAN,
	posix_signal_name VARCHAR(32),

	cpu_usage DOUBLE PRECISION,
	mem_usage DOUBLE PRECISION,
	vmem_usage DOUBLE PRECISION,
	queue VARCHAR(255),
	project VARCHAR(255),
	walltime INTEGER,
	nodes TEXT,
	cancelled BOOLEAN,
	domain VARCHAR(255),
	ext_info TEXT
);

CREATE INDEX jobs_id_idx on jobs(id);
CREATE INDEX jobs_drms_id_idx on jobs(drms_id);
CREATE INDEX jobs_reservation_id_idx on jobs(reservation_id);

CREATE TABLE jobs_acc
(
	serial SERIAL PRIMARY KEY,

	id VARCHAR(36) NOT NULL UNIQUE,
	drms_id VARCHAR(255),

	reservation_id VARCHAR(36),

	submitted_jsdl TEXT NOT NULL,
	jsdl TEXT NOT NULL,

	subject TEXT NOT NULL,
	mapped_user VARCHAR(255) NOT NULL,

	state VARCHAR(16) NOT NULL REFERENCES states(name),
	staging_fault TEXT,

	submission_time TIMESTAMP WITH TIME ZONE NOT NULL,
	start_time TIMESTAMP WITH TIME ZONE,
	end_time TIMESTAMP WITH TIME ZONE,
	CONSTRAINT valid_start_end CHECK (end_time >= start_time),

	exit_status INTEGER,
	core_dumped BOOLEAN,
	posix_signal_name VARCHAR(32),

	cpu_usage DOUBLE PRECISION,
	mem_usage DOUBLE PRECISION,
	vmem_usage DOUBLE PRECISION,
	queue VARCHAR(255),
	project VARCHAR(255),
	walltime INTEGER,
	nodes TEXT,
	cancelled BOOLEAN,
	domain VARCHAR(255),
	ext_info TEXT

);

CREATE INDEX jobs_acc_id_idx on jobs_acc(id);
CREATE INDEX jobs_acc_drms_id_idx on jobs_acc(drms_id);
CREATE INDEX jobs_acc_reservation_id_idx on jobs_acc(reservation_id);

CREATE TABLE reservations
(
	id VARCHAR(36) PRIMARY KEY,
	drms_id VARCHAR(255) NOT NULL,

	submitted_ardl TEXT NOT NULL,
	ardl TEXT NOT NULL,

	subject TEXT NOT NULL,
	mapped_user VARCHAR(255) NOT NULL,

	submission_time TIMESTAMP WITH TIME ZONE NOT NULL,
	start_time TIMESTAMP WITH TIME ZONE NOT NULL,
	end_time TIMESTAMP WITH TIME ZONE NOT NULL,
	nodes TEXT,
	CONSTRAINT valid_start_end CHECK (end_time >= start_time)
);

CREATE INDEX reservations_drms_id_idx on reservations(drms_id);

CREATE TABLE reservations_acc
(
	id VARCHAR(36) PRIMARY KEY,
	drms_id VARCHAR(255) NOT NULL,

	submitted_ardl TEXT NOT NULL,
	ardl TEXT NOT NULL,

	subject TEXT NOT NULL,
	mapped_user VARCHAR(255) NOT NULL,

	submission_time TIMESTAMP WITH TIME ZONE NOT NULL,
	start_time TIMESTAMP WITH TIME ZONE,
	end_time TIMESTAMP WITH TIME ZONE,
	nodes TEXT,
	CONSTRAINT valid_start_end CHECK (end_time >= start_time)
);

CREATE INDEX reservations_acc_drms_id_idx on reservations_acc(drms_id);

CREATE TABLE accounts_pool
(
	account_name VARCHAR(255) PRIMARY KEY,
	home_dir VARCHAR(255) NOT NULL,
	uid INTEGER NOT NULL UNIQUE,
	ref_count INTEGER NOT NULL,
	subject TEXT UNIQUE,
	last_access_time TIMESTAMP WITH TIME ZONE,
	expires_on TIMESTAMP WITH TIME ZONE 
	CONSTRAINT ref_count_non_negative CHECK (ref_count >= 0)
);

CREATE INDEX accounts_pool_subject_idx on accounts_pool(subject);
